part of sam_impl;

class AppServiceImpl extends AppService {
  static const String TAG = "AppServiceImpl";
  final Dio dio;
  final BaseLogger logger;

  AppServiceImpl({required this.dio, required this.logger});

  @override
  Future<UserAccount> getUserAccountByEmail(String email) async {
    try {
      return await dio
          .get(ApiEndpoints.USER,
              queryParameters: {'user_email': email, 'profile': 2})
          .then((value) => UserAccount.fromJson(value.data['body']))
          .catchError((e) {
            logger.error(TAG, e.toString());
            throw 'Failed to parse user account';
          });
    } on DioError catch (_) {
      logger.error(
          TAG,
          getThrowsMessage(
              'Failed to get user account by email', _.response),
          {'email': email});
      throw 'Failed to get user account by email';
    }
  }

  @override
  Future<UserProfile> postNewUser(UserProfile user, String password) async {
    try {
      final String body = json.encode({
        'user_fname': '${user.userFname}',
        'user_lname': '${user.userLname}',
        'user_email': '${user.userEmail}',
        'user_pwd': password,
        'user_phone': ''
      });

      return await dio
          .post(ApiEndpoints.USER, queryParameters: {'profile': 0}, data: body)
          .then((value) {
        if (value.data.containsKey('body')) {
          return UserProfile.fromJson(value.data['body']);
        }
        logger.debug(TAG, "Results not valid postNewUser", {
          'user_fname': '${user.userFname}',
          'user_lname': '${user.userLname}',
          'user_email': '${user.userEmail}'
        });
        throw 'Results not valid';
      }).catchError((e) {
        logger.error(TAG, e.toString());
        throw 'Failed to parse user profile';
      });
    } on DioError catch (_) {
      logger.error(TAG,
          getThrowsMessage('Failed to register new user', _.response), {
        'user_fname': '${user.userFname}',
        'user_lname': '${user.userLname}',
        'user_email': '${user.userEmail}'
      });
      throw getThrowsMessage('Failed to register new user', _.response);
    }
  }

  @override
  Future<bool> putRequestResetPassword(String accountId) async {
    try {
      return await dio.put('${ApiEndpoints.USER}/$accountId',
          queryParameters: {'profile': 5}).then((value) => true);
    } on DioError catch (_) {
      logger.error(
          TAG,
          getThrowsMessage(
              'Failed to request password reset', _.response),
          {'user': accountId});
      throw 'Failed to request password reset';
    }
  }

  @override
  Future<bool> putResetPassword(
      String accountId, String verificationCode, String newPassword) async {
    try {
      final String body = json.encode(
          {'vcode': '$verificationCode', 'user_newpwd': '$newPassword'});

      return await dio
          .put('${ApiEndpoints.USER}/$accountId',
              queryParameters: {'profile': 5}, data: body)
          .then((value) => true);
    } on DioError catch (_) {
      logger.error(
          TAG,
          getThrowsMessage('Failed to reset password', _.response),
          {'user': accountId});
      throw 'Failed to reset password';
    }
  }

  @override
  Future<bool> putVerify(String userId, String verificationCode) async {
    try {
      final String body = json.encode({'vcode': '$verificationCode'});

      return await dio
          .put('${ApiEndpoints.USER}/$userId',
              queryParameters: {'profile': 3}, data: body)
          .then((value) => true);
    } on DioError catch (_) {
      logger.error(
          TAG,
          getThrowsMessage('Failed to verify user', _.response),
          {'user': userId});
      throw 'Failed to verify user';
    }
  }

  @override
  String getThrowsMessage(
      String defaultError, Response? response) {
    if (response != null && response.data != null) {
      try {
        ApiError apiError = ApiError.fromJson(response.data);
        return apiError.desc;
      } catch (e) {}
    }

    return defaultError;
  }
}
